//+------------------------------------------------------------------+
//|                                                Random Mersen.mq5 |
//|                                            Copyright 2013, Urain |
//|                            https://login.mql5.com/ru/users/Urain |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
#property version   "1.00"
#property script_show_inputs
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
/****************************************
* Author C++ ver: Agner Fog
* Date created:   1997
* Last modified:  2008-11-16
* Source URL:     www.agner.org/random
*///*************************************

class CRandomMersenne
  {                // Encapsulate random number generator
   // Choose which version of Mersenne Twister you want:
/*-----------#if 0 
// Define constants for type MT11213A:
#define MERS_N   351
#define MERS_M   175
#define MERS_R   19
#define MERS_U   11
#define MERS_S   7
#define MERS_T   15
#define MERS_L   17
#define MERS_A   0xE4BD75F5
#define MERS_B   0x655E5280
#define MERS_C   0xFFD58000
*///-----------#else    
// or constants for type MT19937:                                                                                                                                                                                                                                                                                                          // or constants for type MT19937:
#define MERS_N   624
#define MERS_M   397
#define MERS_R   31
#define MERS_U   11
#define MERS_S   7
#define MERS_T   15
#define MERS_L   18
#define MERS_A   0x9908B0DF
#define MERS_B   0x9D2C5680
#define MERS_C   0xEFC60000
   //-----------#endif

public:
                     CRandomMersenne(int seed){RandomInit(seed); LastInterval=0;};
                    ~CRandomMersenne(void){};
   void              RandomInit(int seed);         // Re-seed
   void              RandomInitByArray(const int &seeds[],int NumSeeds); // Seed by more than 32 bits
   int               IRandom(int min,int max);     // Output random integer
   int               IRandomX(int min,int max,bool multi=true);    // Output random integer, exact
   double            Random();                     // Output random float
   uint              BRandom();                    // Output random bits
private:
   void              Init0(int seed);              // Basic initialization procedure
   uint              mt[MERS_N];                   // State vector
   int               mti;                          // Index into mt
   uint              LastInterval;                 // Last interval length for IRandomX
   uint              RLimit;                       // Rejection limit used by IRandomX
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CRandomMersenne::Init0(int seed)
  {
// Seed generator
   const uint factor=(ulong)1812433253;
   mt[0]=seed;
   for(mti=1; mti<MERS_N; mti++)
     {
      mt[mti]=(factor *(mt[mti-1]^(mt[mti-1]>>30))+mti);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CRandomMersenne::RandomInit(int seed)
  {
// Initialize and seed
   Init0(seed);

// Randomize some more
   for(int i=0; i<37; i++) BRandom();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CRandomMersenne::RandomInitByArray(const int &seeds[],int NumSeeds)
  {
// Seed by more than 32 bits
   int i,j,k;

// Initialize
   Init0(19650218);

   if(NumSeeds<=0) return;

// Randomize mt[] using whole seeds[] array
   i = 1;  j = 0;
   k = (MERS_N > NumSeeds ? MERS_N : NumSeeds);
   for(; k; k--)
     {
      mt[i]=(mt[i]^((mt[i-1]^(mt[i-1]>>30))*(ulong)1664525))+(uint)seeds[j]+j;
      i++; j++;
      if(i>=MERS_N) {mt[0]=mt[MERS_N-1]; i=1;}
      if(j>=NumSeeds) j=0;
     }
   for(k=MERS_N-1; k; k--)
     {
      mt[i]=(mt[i]^((mt[i-1]^(mt[i-1]>>30))*(ulong)1566083941))-i;
      if(++i>=MERS_N) {mt[0]=mt[MERS_N-1]; i=1;}
     }
   mt[0]=(ulong)0x80000000;  // MSB is 1; assuring non-zero initial array

                             // Randomize some more
   mti=0;
   for(int i=0; i<=MERS_N; i++) BRandom();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
uint CRandomMersenne::BRandom()
  {
// Generate 32 random bits
   uint y;

   if(mti>=MERS_N)
     {
      // Generate MERS_N words at one time
      const uint LOWER_MASK=((ulong)1<<MERS_R)-1;       // Lower MERS_R bits
      const uint UPPER_MASK=0xFFFFFFFF<<MERS_R;      // Upper (32 - MERS_R) bits
      static const uint mag01[2]={0,MERS_A};

      int kk;
      for(kk=0; kk<MERS_N-MERS_M; kk++)
        {
         y=(mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
         mt[kk]=mt[kk+MERS_M]^(y>>1)^mag01[y  &1];
        }

      for(; kk<MERS_N-1; kk++)
        {
         y=(mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
         mt[kk]=mt[kk+(MERS_M-MERS_N)]^(y>>1)^mag01[y  &1];
        }

      y=(mt[MERS_N-1]&UPPER_MASK)|(mt[0]&LOWER_MASK);
      mt[MERS_N-1]=mt[MERS_M-1]^(y>>1)^mag01[y  &1];
      mti=0;
     }
   y=mt[mti++];

// Tempering (May be omitted):
   y ^=  y >> MERS_U;
   y ^= (y << MERS_S) & MERS_B;
   y ^= (y << MERS_T) & MERS_C;
   y ^=  y >> MERS_L;

   return y;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CRandomMersenne::Random()
  {
// Output random float number in the interval 0 <= x < 1
// Multiply by 2^(-32)
   return(double)BRandom() *(1./(65536.*65536.));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CRandomMersenne::IRandom(int min,int max)
  {
// Output random integer in the interval min <= x <= max
// Relative error on frequencies < 2^-32
   if(max<=min)
     {
      if(max==min) return min; else return 0x80000000;
     }
// Multiply interval with random and truncate
   int r=int((double)(uint)(max-min+1)*Random()+min);
   if(r>max) r=max;
   return r;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CRandomMersenne::IRandomX(int min,int max,bool multi=true)
  {
// Output random integer in the interval min <= x <= max
// Each output value has exactly the same probability.
// This is obtained by rejecting certain bit values so that the number
// of possible bit values is divisible by the interval length
   if(max<=min)
     {
      if(max==min) return min; else return 0x80000000;
     }
   if(multi)
     {
      // 64 bit integers available. Use multiply and shift method
      uint interval;                    // Length of interval
      ulong longran;                     // Random bits * interval
      uint iran;                        // Longran / 2^32
      uint remainder;                   // Longran % 2^32

      interval=uint(max-min+1);
      if(interval!=LastInterval)
        {
         // Interval length has changed. Must calculate rejection limit
         // Reject when remainder >= 2^32 / interval * interval
         // RLimit will be 0 if interval is a power of 2. No rejection then
         RLimit=uint(((ulong)1<<32)/interval)*interval-1;
         LastInterval=interval;
        }
      do
        { // Rejection loop
         longran=(ulong)BRandom()*interval;
         iran=(uint)(longran>>32);
         remainder=(uint)longran;
        }
      while(remainder>RLimit);
      // Convert back to signed and return result
      return(int)iran+min;
     }
   else
     {
      // 64 bit integers not available. Use modulo method
      uint interval;                    // Length of interval
      uint bran;                        // Random bits
      uint iran;                        // bran / interval
      uint remainder;                   // bran % interval

      interval=uint(max-min+1);
      if(interval!=LastInterval)
        {
         // Interval length has changed. Must calculate rejection limit
         // Reject when iran = 2^32 / interval
         // We can't make 2^32 so we use 2^32-1 and correct afterwards
         RLimit=(uint)0xFFFFFFFF/interval;
         if((uint)0xFFFFFFFF%interval==interval-1) RLimit++;
        }
      do
        { // Rejection loop
         bran = BRandom();
         iran = bran / interval;
         remainder=bran%interval;
        }
      while(iran>=RLimit);
      // Convert back to signed and return result
      return(int)remainder+min;
     }
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CRandomMother
  {                  // Encapsulate random number generator
public:
   void              RandomInit(int seed);          // Initialization
   int               IRandom(int min,int max);      // Get integer random number in desired interval
   double            Random();                    // Get floating point random number
   uint              BRandom();                 // Output random bits
                     CRandomMother(int seed){RandomInit(seed);};
                    ~CRandomMother(void){};
protected:
   uint              x[5];                      // History buffer
  };
// Output random bits
uint CRandomMother::BRandom()
  {
   ulong sum;
   sum=(ulong)2111111111 *(ulong)x[3]+
       (ulong)1492 *(ulong)(x[2])+
       (ulong)1776 *(ulong)(x[1])+
       (ulong)5115 *(ulong)(x[0])+
       (ulong)x[4];
   x[3] = x[2];  x[2] = x[1];  x[1] = x[0];
   x[4] = (uint)(sum >> 32);                  // Carry
   x[0] = (uint)sum;                          // Low 32 bits of sum
   return x[0];
  }
// returns a random number between 0 and 1:
double CRandomMother::Random()
  {
   return(double)BRandom() *(1./(65536.*65536.));
  }
// returns integer random number in desired interval:
int CRandomMother::IRandom(int min,int max)
  {
// Output random integer in the interval min <= x <= max
// Relative error on frequencies < 2^-32
   if(max<=min)
     {
      if(max==min) return min; else return 0x80000000;
     }
// Assume 64 bit integers supported. Use multiply and shift method
   uint interval;                  // Length of interval
   ulong longran;                   // Random bits * interval
   uint iran;                      // Longran / 2^32

   interval = (uint)(max - min + 1);
   longran  = (ulong)BRandom() * interval;
   iran=(uint)(longran>>32);
// Convert back to signed and return result
   return(int)iran+min;
  }
// this function initializes the random number generator:
void CRandomMother::RandomInit(int seed)
  {
   int i;
   uint s=seed;
// make random numbers and put them into the buffer
   for(i=0; i<5; i++)
     {
      s=s*29943829-1;
      x[i]=s;
     }
// randomize some more
   for(i=0; i<19; i++) BRandom();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

//--- input parameters
input int      Seed=245845;
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
//int seed = (int)GetTickCount();            // random seed

// choose one of the random number generators: 
  
// CRandomMersenne *RanGen=new CRandomMersenne(Seed);
   CRandomMother *RanGen=new CRandomMother(Seed);       // make instance of random number generator

   int i;                              // loop counter
   int ir;                             // random integer number
   double fr;                          // random floating point number

//--- make random integers in interval from 0 to 99, inclusive:
   printf("\nRandom integers in interval from 0 to 99:");
   for(i=0; i<40; i++)
     {
      ir=RanGen.IRandom(0,99);
      printf("%6i  ",ir);
     }

//--- make random floating point numbers in interval from 0 to 1:
   printf("\nRandom floating point numbers in interval from 0 to 1:");
   for(i=0; i<32; i++)
     {
      fr=RanGen.Random();
      printf("%8.6f  ",fr);
     }

//--- make random bits
   printf("\nRandom bits (hexadecimal):");
   for(i=0; i<32; i++)
     {
      ir=RanGen.BRandom();
      printf("%08lX  ",ir);
     }
   delete RanGen;
  }
//+------------------------------------------------------------------+
